==> Goober_0_3_0.zip: goober-ionic/src/pages/stream/stream.ts <==
import { Component, ViewChild, ChangeDetectorRef } from '@angular/core';
import { ViewController, NavController, NavParams, ModalController, Content, ToastController, PopoverController } from 'ionic-angular';
import { ThreadPage } from '../thread/thread';

import * as pnut from 'pnut-butter';

/**
 * Generated class for the StreamPage page.
 *
 * See http://ionicframework.com/docs/components/#navigation for more info
 * on Ionic pages and navigation.
 */
@Component({
  selector: 'page-stream',
  templateUrl: 'stream.html',
})
export class StreamPage {

  @ViewChild(Content) content: Content;

  title: string;
  posts: Array<Object> = [];
  since_id: string;
  before_id: string;
  fetcher: any;
  fcaller: any;
  myUsername: string;
  showScrollBtn: boolean = false;

  constructor(public navCtrl: NavController, public navParams: NavParams, public modalCtrl: ModalController,
    private changeDetectorRef: ChangeDetectorRef, public toastCtrl: ToastController, public popoverCtrl: PopoverController) {
    // console.log(JSON.stringify(navParams));

    switch (navParams.data.stream) {
      case 'global':
        this.title = 'Global';
        this.fetcher = pnut.global;
        this.fetchPosts();
        break;
      case 'personal':
        this.title = 'Timeline';
        this.fetcher = pnut.personal;
        this.fetchPosts();
        break;
      case 'mentions':
        this.title = 'Mentions';
        this.fetcher = pnut.mentions;
        this.fetchMyPosts();
        break;
      case 'bookmarks':
        this.title = 'Bookmarks';
        this.fetcher = pnut.bookmarks;
        this.fetchMyPosts();
        break;
    }
    pnut.user('me').then(res => {
      this.myUsername = res.data.username;
    }).catch(err => {
      console.log(err);
    });
  }

  ngAfterViewInit() {
    this.content.ionScroll.subscribe((event) => {
      // console.log('scrolling ', event);
      if (event.scrollTop > 0) {
        this.showScrollBtn = true;
      } else {
        this.showScrollBtn = false;
      }
      this.changeDetectorRef.detectChanges();
    });
  }

  fetchOlderPosts(infiniteScroll) {
    let params = {include_raw: 1, include_reposted_by:1, before_id: this.before_id, count: 40};
    if (this.title === 'Mentions') {
      this.fcaller = this.fetcher('me', params);
    } else {
      this.fcaller = this.fetcher(params);
    }
    this.fcaller.then(res => {
      if (res.data.length > 0) {
        for (var i = 0; i < res.data.length; i++) {
          if (res.data[i]['repost_of']) {
            res.data[i] = res.data[i]['repost_of']
          }
          for (var j = 0; j < res.data[i]['content']['entities']['mentions'].length; j++) {
            var men = res.data[i]['content']['entities']['mentions'][j]['text'];
            if (this.myUsername === men) {
              res.data[i]['you_are_mentioned'] = true;
            }
          }
        }
        this.posts.push.apply(this.posts, res.data);
        this.before_id = res.meta.min_id;
      }
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
      infiniteScroll.complete();
    }).catch(err => {
      console.log(err);
    });
  }

  fetchNewerPosts(refresher) {
    let params = {include_raw: 1, include_reposted_by: 1, since_id: this.since_id, count: 40};
    if (this.title === 'Mentions') {
      this.fcaller = this.fetcher('me', params);
    } else {
      this.fcaller = this.fetcher(params);
    }
    this.fcaller.then(res => {
      if (res.data.length > 0) {
        for (var i = 0; i < res.data.length; i++) {
          if (res.data[i]['repost_of']) {
            res.data[i] = res.data[i]['repost_of']
          }
          for (var j = 0; j < res.data[i]['content']['entities']['mentions'].length; j++) {
            var men = res.data[i]['content']['entities']['mentions'][j]['text'];
            if (this.myUsername === men) {
              res.data[i]['you_are_mentioned'] = true;
            }
          }
        }
        Array.prototype.unshift.apply(this.posts, res.data);
        this.since_id = res.meta.max_id;
      }
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
      refresher.complete();
    }).catch(err => {
      console.log(err);
    });
  }

  fetchPosts() {
    this.fetcher({include_raw: 1, include_reposted_by: 1, count: 40}).then(res => {
      for (var i = 0; i < res.data.length; i++) {
        if (res.data[i]['repost_of']) {
          res.data[i] = res.data[i]['repost_of']
          var reposted_by_string = "";
          for (var j = 0; j < res.data[i]['reposted_by'].length; j++) {
            reposted_by_string = reposted_by_string + res.data[i]['reposted_by'][j]['username'] + ", ";
          }
          // res.data[i]['reposted_by_string'] = "Reposted by: " + reposted_by_string;
        }
        if (res.data[i].content) {
          for (var j = 0; j < res.data[i]['content']['entities']['mentions'].length; j++) {
            var men = res.data[i]['content']['entities']['mentions'][j]['text'];
            if (this.myUsername === men) {
              res.data[i]['you_are_mentioned'] = true;
            }
          }
        }
      }
      this.posts = res.data;
      this.since_id = res.meta.max_id;
      this.before_id = res.meta.min_id;
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
    }).catch(err => {
      console.log(err);
    });
  }

  fetchThread(threadid) {
    pnut.thread(threadid, {include_raw: 1, count: 140}).then(res => {
      this.navCtrl.push(ThreadPage, {posts: res.data});
    }).catch(err => {
      console.log(err);
    });
  }

  fetchMyPosts() {
    console.log('-- fetching mentions --');
    this.fetcher('me', {include_raw: 1, count: 40}).then(res => {
      this.posts = res.data;
      this.since_id = res.meta.max_id;
      this.before_id = res.meta.min_id;
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
    }).catch(err => {
      console.log(err);
    });
  }

  bookmark(postid, bookmarked) {
    if (bookmarked) {
      pnut.deleteBookmark(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Bookmark updated.");
      }).catch(err => {
        console.log(err);
      });
    } else {
      pnut.bookmark(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Bookmark updated.");
      }).catch(err => {
        console.log(err);
      });
    }
  }

  repost(postid, reposted) {
    if (reposted) {
      pnut.deleteRepost(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Repost updated.");
      }).catch(err => {
        console.log(err);
      });
    } else {
      pnut.repost(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Repost updated.");
      }).catch(err => {
        console.log(err);
      });
    }
  }

  updatePost(postid) {
    pnut.post(postid, {include_raw: 1}).then(res => {
      for (var i = 0; i < this.posts.length; i++) {
        if (this.posts[i]['id'] === postid) {
          this.posts[i] = res.data;
          break;
        }
      }
    }).catch(err => {
      console.log(err);
    });
  }

  showNewPost() {
    let newPostModal = this.modalCtrl.create(NewPostModal);
    newPostModal.present();
  }

  showReplyPost(postData) {
    let newPostModal = this.modalCtrl.create(NewPostModal, {type: 'reply', post: postData, me: this.myUsername});
    newPostModal.present();
  }

  showQuotedPost(postData) {
    console.log(postData);
    let newPostModal = this.modalCtrl.create(NewPostModal, {type: 'quote', post: postData, me: this.myUsername});
    newPostModal.present();
  }

  presentToast(text) {
    let toast = this.toastCtrl.create({
      position: 'top',
      message: text,
      duration: 2000
    });
    toast.present();
  }

  presentPostMenu(myEvent) {
    let popover = this.popoverCtrl.create(PostMenu);
    popover.present({
      ev: myEvent
    });
  }

  scrollToTop() {
    this.content.scrollToTop();
  }

}

@Component({
  // templateUrl: 'new-post.html'
  selector: 'modal-newpost',
  template: `
<ion-header>
  <ion-toolbar>

    <ion-buttons start>
      <button ion-button (click)="dismiss()">
        <span ion-text color="primary">Cancel</span>
      </button>
    </ion-buttons>

    <ion-title>{{ title }}</ion-title>

  </ion-toolbar>
</ion-header>

<ion-content>
  <ion-card>
    <ion-card-content>
      <ion-item>
        <ion-textarea [(ngModel)]="ptext" autocomplete="true" spellcheck="true" clearInput="true" rows="8"></ion-textarea>
      </ion-item>
    </ion-card-content>
    <ion-row justify-content-end>
      <ion-col col-2><div text-center>{{254 - ptext.length}}</div></ion-col>
      <ion-col col-2>
        <button ion-button (click)="send()">
          <ion-icon name="send"></ion-icon>
        </button>
      </ion-col>
    </ion-row>
  </ion-card>
</ion-content>
`
})
export class NewPostModal {
  title: string;
  replyid: string;
  ptext: string = "";
  options: Object;
  myUsername: string;

  constructor(public navParams: NavParams, public viewCtrl: ViewController, public toastCtrl: ToastController) {
    console.log(this.navParams);
    this.myUsername = navParams.data.me;
    if (navParams.data.type === 'reply') {
      this.replyid = navParams.data.post.id;
      this.options = {replyTo: this.replyid};
      this.ptext = "@" + navParams.data.post.user.username + " ";
      if (navParams.data.post.content.entities) {
        if (navParams.data.post.content.entities.mentions) {
          this.ptext = this.ptext + this.parseMentions(navParams.data.post.content.entities.mentions);
        }
      }
      this.title = "Reply to " + navParams.data.post.user.username
    } else if (navParams.data.type === 'quote') {
      this.ptext = " >> @" + navParams.data.post.user.username + ": " + navParams.data.post.content.text;
      this.title = "New Post";
    } else {
      this.title = "New Post";
    }
  }

  dismiss() {
   this.viewCtrl.dismiss();
  }

  send() {
    console.log(this.ptext);
    pnut.createPost(this.ptext, this.options).then(res => {
      console.log(res);
      this.presentToast("Status posted.");
    }).catch(err => {
      console.log(err);
    });
    this.viewCtrl.dismiss();
  }

  presentToast(text) {
    let toast = this.toastCtrl.create({
      position: 'top',
      message: text,
      duration: 2000
    });
    toast.present();
  }

  parseMentions(mentions) {
    let mtext = ""
    for(var i = 0; i < mentions.length; i++) {
      let mu = mentions[i].text;
      if (mu !== this.myUsername) {
        mtext += "@" + mu + " ";
      }
    }
    if (mtext.length > 0) {
      return mtext;
    } else {
      return "";
    }
  }
}

@Component({
  template: `
<ion-list>
  <button ion-item disabled (click)="close()">Share</button>
  <button ion-item *ngIf="showDelBtn" (click)="close()">Delete</button>
</ion-list>
`
})
export class PostMenu {

  showDelBtn: boolean = false;

  constructor(public navParams: NavParams, public viewCtrl: ViewController) {}

  // buttonState() {
  //   return this.showDelBtn ? _.state : _;
  // }

  close() {
    this.viewCtrl.dismiss();
  }
}

==> Goober_0_5_0.zip: goober-ionic/src/pages/stream/stream.ts <==
import { Component, ViewChild, ChangeDetectorRef } from '@angular/core';
import { ViewController, NavController, NavParams, ModalController, Content, ToastController, PopoverController } from 'ionic-angular';
import { ThreadPage } from '../thread/thread';
import { Storage } from '@ionic/storage';
import { Events } from 'ionic-angular';

import * as pnut from 'pnut-butter';

/**
 * Generated class for the StreamPage page.
 *
 * See http://ionicframework.com/docs/components/#navigation for more info
 * on Ionic pages and navigation.
 */
@Component({
  selector: 'page-stream',
  templateUrl: 'stream.html',
})
export class StreamPage {

  @ViewChild(Content) content: Content;

  title: string;
  posts: Array<Object> = [];
  since_id: string;
  before_id: string;
  fetcher: any;
  fcaller: any;
  myUsername: string;
  showScrollBtn: boolean = false;
  showUnified: boolean;
  ccOnReply: boolean = false;

  constructor(public navCtrl: NavController, public navParams: NavParams, public modalCtrl: ModalController,
    private changeDetectorRef: ChangeDetectorRef, public toastCtrl: ToastController, private storage: Storage,
    public popoverCtrl: PopoverController, public events: Events) {
    // console.log(JSON.stringify(navParams));

    this.storage.get('cc').then((val) => {
      this.ccOnReply = val;
    });

    this.storage.get('unified').then((val) => {
      this.showUnified = val;

      switch (navParams.data.stream) {
        case 'global':
          this.title = 'Global';
          this.fetcher = pnut.global;
          this.fetchPosts();
          break;
        case 'personal':
          this.title = 'Timeline';
          console.log(this.showUnified);
          this.fetcher = this.showUnified ? pnut.unified : pnut.personal;
          this.fetchPosts();
          break;
        case 'mentions':
          this.title = 'Mentions';
          this.fetcher = pnut.mentions;
          this.fetchMyPosts();
          break;
        case 'bookmarks':
          this.title = 'Bookmarks';
          this.fetcher = pnut.bookmarks;
          this.fetchMyPosts();
          break;
      }

    }).catch(err => {
      console.log('ERROR: ' + err);
    });


    pnut.user('me').then(res => {
      this.myUsername = res.data.username;
    }).catch(err => {
      // console.log('-*-');
      // console.log(JSON.stringify(err));
    });

    this.events.subscribe('stream:reload', (event) => {
      console.log('-reload-');
      this.refreshPage();
    });

  }

  ngAfterViewInit() {
    this.content.ionScroll.subscribe((event) => {
      // console.log('scrolling ', event);
      if (event.scrollTop > 0) {
        this.showScrollBtn = true;
      } else {
        this.showScrollBtn = false;
      }
      this.changeDetectorRef.detectChanges();
    });
  }

  refreshPage() {
    // this.navCtrl.setRoot(this.navCtrl.getActive().component);
    this.navCtrl.setRoot(StreamPage, {stream: 'personal'});
  }

  fetchOlderPosts(infiniteScroll) {
    let params = {
      include_deleted: 0,
      include_raw: 1,
      include_reposted_by:1,
      before_id: this.before_id,
      count: 40
    };
    if (this.title === 'Mentions') {
      this.fcaller = this.fetcher('me', params);
    } else {
      this.fcaller = this.fetcher(params);
    }
    this.fcaller.then(res => {
      if (res.data.length > 0) {
        this.posts.push.apply(this.posts, this.parseData(res.data));
        this.before_id = res.meta.min_id;
      }
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
      infiniteScroll.complete();
    }).catch(err => {
      console.log(err);
    });
  }

  fetchNewerPosts(refresher) {
    let params = {
      include_deleted: 0,
      include_raw: 1,
      include_reposted_by: 1,
      since_id: this.since_id,
      count: 40
    };
    if (this.title === 'Mentions') {
      this.fcaller = this.fetcher('me', params);
    } else {
      this.fcaller = this.fetcher(params);
    }
    this.fcaller.then(res => {
      if (res.data.length > 0) {
        Array.prototype.unshift.apply(this.posts, this.parseData(res.data));
        this.since_id = res.meta.max_id;
      }
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
      refresher.complete();
    }).catch(err => {
      console.log(err);
    });
  }

  fetchPosts() {
    let params = {
      include_deleted: 0,
      include_raw: 1,
      include_reposted_by: 1,
      count: 40
    };
    this.fetcher(params).then(res => {
      this.posts = this.parseData(res.data);
      this.since_id = res.meta.max_id;
      this.before_id = res.meta.min_id;
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
    }).catch(err => {
      console.log(err);
    });
  }

  parseData(data) {
    var pdata = [];
    for (var i = 0; i < data.length; i++) {
      if (!data[i].is_deleted) {
        if (data[i]['repost_of']) {
          data[i] = data[i]['repost_of']
          var reposted_by_string = "";
          for (var j = 0; j < data[i]['reposted_by'].length; j++) {
            reposted_by_string = reposted_by_string + data[i]['reposted_by'][j]['username'] + ", ";
          }
          // data[i]['reposted_by_string'] = "Reposted by: " + reposted_by_string;
        }
        if (data[i].content) {
          for (var j = 0; j < data[i]['content']['entities']['mentions'].length; j++) {
            var men = data[i]['content']['entities']['mentions'][j]['text'];
            if (this.myUsername === men) {
              data[i]['you_are_mentioned'] = true;
            }
          }
        }
        pdata.push(data[i]);
      }
    }
    return pdata;
  }

  fetchThread(threadid) {
    pnut.thread(threadid, {include_deleted: 0, include_raw: 1, count: 140}).then(res => {
      this.navCtrl.push(ThreadPage, {posts: res.data, me: this.myUsername});
    }).catch(err => {
      console.log(err);
    });
  }

  fetchMyPosts() {
    console.log('-- fetching mentions --');
    this.fetcher('me', {include_raw: 1, count: 40}).then(res => {
      this.posts = res.data;
      this.since_id = res.meta.max_id;
      this.before_id = res.meta.min_id;
      console.log('since_id: ' + this.since_id);
      console.log('before_id: ' + this.before_id);
    }).catch(err => {
      console.log(err);
    });
  }

  bookmark(postid, bookmarked) {
    if (bookmarked) {
      pnut.deleteBookmark(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Bookmark updated.");
      }).catch(err => {
        console.log(err);
      });
    } else {
      pnut.bookmark(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Bookmark updated.");
      }).catch(err => {
        console.log(err);
      });
    }
  }

  repost(postid, reposted) {
    if (reposted) {
      pnut.deleteRepost(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Repost updated.");
      }).catch(err => {
        console.log(err);
      });
    } else {
      pnut.repost(postid).then(res => {
        console.log(res);
        this.updatePost(res.data.id);
        this.presentToast("Repost updated.");
      }).catch(err => {
        console.log(err);
      });
    }
  }

  updatePost(postid) {
    pnut.post(postid, {include_raw: 1}).then(res => {
      for (var i = 0; i < this.posts.length; i++) {
        if (this.posts[i]['id'] === postid) {
          this.posts[i] = res.data;
          break;
        }
      }
    }).catch(err => {
      console.log(err);
    });
  }

  showNewPost() {
    let newPostModal = this.modalCtrl.create(NewPostModal);
    newPostModal.present();
  }

  showReplyPost(postData) {
    let newPostModal = this.modalCtrl.create(NewPostModal, {type: 'reply', post: postData, me: this.myUsername, cc: this.ccOnReply});
    newPostModal.present();
  }

  showQuotedPost(postData) {
    console.log(postData);
    let newPostModal = this.modalCtrl.create(NewPostModal, {type: 'quote', post: postData, me: this.myUsername, cc: this.ccOnReply});
    newPostModal.present();
  }

  presentToast(text) {
    let toast = this.toastCtrl.create({
      position: 'top',
      message: text,
      duration: 2000
    });
    toast.present();
  }

  presentPostMenu(myEvent, postData) {
    let popover = this.popoverCtrl.create(PostMenu, {post: postData, me: this.myUsername});
    popover.present({ev: myEvent});
  }

  scrollToTop() {
    this.content.scrollToTop();
  }

}

@Component({
  // templateUrl: 'new-post.html'
  selector: 'modal-newpost',
  template: `
<ion-header>
  <ion-toolbar>

    <ion-buttons start>
      <button ion-button (click)="dismiss()">
        <span ion-text color="primary">Cancel</span>
      </button>
    </ion-buttons>

    <ion-title>{{ title }}</ion-title>

  </ion-toolbar>
</ion-header>

<ion-content>
  <ion-card>
    <ion-card-content>
      <ion-item>
        <ion-textarea [(ngModel)]="ptext" autocomplete="true" spellcheck="true" clearInput="true" rows="8"></ion-textarea>
      </ion-item>
    </ion-card-content>
    <ion-row justify-content-end>
      <ion-col col-2><div text-center>{{254 - ptext.length}}</div></ion-col>
      <ion-col col-2>
        <button ion-button (click)="send()">
          <ion-icon name="send"></ion-icon>
        </button>
      </ion-col>
    </ion-row>
  </ion-card>
</ion-content>
`
})
export class NewPostModal {
  title: string;
  replyid: string;
  ptext: string = "";
  options: Object;
  myUsername: string;

  constructor(public navParams: NavParams, public viewCtrl: ViewController, public toastCtrl: ToastController,
    public events: Events) {
    console.log(this.navParams);
    this.myUsername = navParams.data.me;
    if (navParams.data.type === 'reply') {
      this.replyid = navParams.data.post.id;
      this.options = {replyTo: this.replyid};
      if (navParams.data.post.user.username !== this.myUsername) {
        this.ptext = "@" + navParams.data.post.user.username + " ";
      } else {
        this.ptext = ""
      }
      if (navParams.data.post.content.entities) {
        if (navParams.data.post.content.entities.mentions.length > 0) {
          this.ptext = this.ptext + this.parseMentions(navParams.data.post.content.entities.mentions);
        }
      }
      this.title = "Reply to " + navParams.data.post.user.username
    } else if (navParams.data.type === 'quote') {
      this.ptext = " >> @" + navParams.data.post.user.username + ": " + navParams.data.post.content.text;
      this.title = "New Post";
    } else {
      this.title = "New Post";
    }
  }

  dismiss() {
   this.viewCtrl.dismiss();
  }

  send() {
    console.log(this.ptext);
    pnut.createPost(this.ptext, this.options).then(res => {
      console.log(res);
      this.presentToast("Status posted.");
      this.events.publish('stream:reload', {});
    }).catch(err => {
      console.log(err);
    });
    this.viewCtrl.dismiss();
  }

  presentToast(text) {
    let toast = this.toastCtrl.create({
      position: 'top',
      message: text,
      duration: 2000
    });
    toast.present();
  }

  parseMentions(mentions) {
    let mtext = ""
    for(var i = 0; i < mentions.length; i++) {
      let mu = mentions[i].text;
      if (mu !== this.myUsername) {
        mtext += "@" + mu + " ";
      }
    }
    if (mtext.length > 0) {
      if (this.navParams.data.cc) {
        mtext = "\n/" + mtext;
      }
      return mtext;
    } else {
      return "";
    }
  }
}

@Component({
  template: `
<ion-list>
  <button ion-item (click)="share()">Share</button>
  <button ion-item *ngIf="showDelBtn" (click)="delete()">Delete</button>
</ion-list>
`
})
export class PostMenu {

  showDelBtn: boolean = false;

  constructor(public navParams: NavParams, public viewCtrl: ViewController, public toastCtrl: ToastController,
    public events: Events) {
    if (navParams.data.me == navParams.data.post.user.username) {
      this.showDelBtn = true;
    } else {
      this.showDelBtn = false;
    }
  }

  share() {
    (<any>window).shareContentPlugin.share(this.navParams.data.post.content.text, function(e) {
      console.log('sharing post:');
      console.log(JSON.stringify(e));
    }, function(e) {
      console.log('sharing failed:');
      console.log(JSON.stringify(e));
    });
    this.close();
  }

  delete() {
    pnut.deletePost(this.navParams.data.post.id).then(res => {
      console.log(res);
      this.presentToast('Post Deleted');
      this.events.publish('stream:reload', {});
    }).catch( err => {
      console.log('-error-');
      console.log(err);
    });
    this.close()
  }

  presentToast(text) {
    let toast = this.toastCtrl.create({
      position: 'top',
      message: text,
      duration: 2000
    });
    toast.present();
  }

  close() {
    this.viewCtrl.dismiss();
  }
}
